#ifndef SF_HELP_H
#define SF_HELP_H

#include <crtdbg.h>
#include <vector>


//////////////////////////////////////////////////////////////////
// CHolder<T>

template <class T>
class CHolder
{
    std::vector<T>  m_vData;
    UINT            m_nPos;

public:
    CHolder() throw();
    ~CHolder() throw();

    UINT AllocIndex();   // can throw bad_alloc or access violation
    void SetAt(UINT nIdx, const T& data) throw();

    const T& operator[](UINT nIdx) const throw();
    UINT size() const throw();

private:
    void UpdatePos() throw();
};


//////////////////////////////////////////////////////////////////
// CReceiver

class CReceiver : private CAuxThunk<CReceiver>
{
    UINT            m_nID;
    CTimerHost&     m_Father;
    SafeTimerProc   m_pfn;

protected:
    friend class CTimerHost;

    CReceiver(CTimerHost& rFather, SafeTimerProc pfnCallback) throw();
    ~CReceiver() throw();

public:
    void OnStop() throw();
    void OnRecycle(UINT nID) throw();
    operator TIMERPROC() const throw();

    void TimerProc(HWND hwnd, UINT uMsg, UINT_PTR idEvent, DWORD dwTime) throw();
};


/**************************************************************************************
                                 Implementations
**************************************************************************************/


///////////////////////////////////////////////////////////////////////////////////////
// CHolder<T>

template <class T>
inline CHolder<T>::CHolder() throw()
    :   m_nPos(0)
{
}

template <class T>
inline CHolder<T>::~CHolder() throw()
{
}

template <class T>
inline const T& CHolder<T>::operator[](UINT nIdx) const throw()
{
    return m_vData[nIdx];
}

template <class T>
inline UINT CHolder<T>::size() const throw()
{
    return m_vData.size();
}

template <class T>
inline UINT CHolder<T>::AllocIndex()
{
    if (m_nPos == m_vData.size()) m_vData.push_back(T());

    return m_nPos;
}

template <class T>
inline void CHolder<T>::SetAt(UINT nIdx, const T& data) throw()
{
    m_vData[nIdx] = data;

    if (data == T())
    {
        if (nIdx < m_nPos) m_nPos = nIdx;
    }
    else
    {
        if (nIdx == m_nPos) UpdatePos();
    }
}

template <class T>
inline void CHolder<T>::UpdatePos() throw()
{
    while(m_nPos != m_vData.size())
    {
        if (m_vData[m_nPos] == T()) break;
        ++m_nPos;
    }
}


//////////////////////////////////////////////////////////////////
// CReceiver

inline CReceiver::CReceiver(CTimerHost& rFather, SafeTimerProc pfnCallback) throw()
    :   m_Father(rFather),
        m_pfn(pfnCallback),
        m_nID((UINT)-1)
{
    InitThunk((TMFP)TimerProc, this);
}

inline CReceiver::~CReceiver() throw()
{
}

inline void CReceiver::OnStop() throw()
{
    _ASSERTE(m_pfn != NULL);
    m_pfn = NULL;
}

inline void CReceiver::OnRecycle(UINT nID) throw()
{
    _ASSERTE(m_pfn == NULL);
    m_nID = nID;
}

inline CReceiver::operator TIMERPROC() const throw()
{
    return reinterpret_cast<TIMERPROC>(GetThunk());
}

#endif //SF_HELP_H
