#include "stdafx.h"
#include "sf_timer.h"
#include "tchar.h"

//////////////////////////////////////////////////////////////////
// CReceiver

void CReceiver::TimerProc(HWND hwnd, UINT uMsg, UINT_PTR idEvent, DWORD dwTime)
{
    if (m_pfn != NULL)
    {
        _ASSERTE(idEvent != BAD_TIMER_ID);
        (*m_pfn)(dwTime);
    }
    else if (idEvent == BAD_TIMER_ID)
    {
        m_Father.RemoveReceiver(m_nID);
        delete this;
    }
}


//////////////////////////////////////////////////////////////////
// CTimerHost

CTimerHost::CTimerHost() throw(CError)
{
    m_hWnd = CreateWindow(_T("STATIC"), _T("SafeTimerWnd"), 0, 0, 0, 0, 0, NULL, NULL, NULL, NULL);
    if (m_hWnd == NULL)
    {
        throw CError();
    }
}

CTimerHost::~CTimerHost() throw()
{
    BOOL bRes = DestroyWindow(m_hWnd);
    _ASSERTE(bRes);

    UINT i;
    for(i = 0; i < m_vReceiver.size(); ++i) delete m_vReceiver[i];
    for(i = 0; i < m_vRecycle.size();  ++i) delete m_vRecycle[i] ;
}

UINT CTimerHost::SetTimer(UINT nElapse, SafeTimerProc pfnCallback) throw()
{
    UINT nTimerID;
    CReceiver* pReceiver;
    try
    {
        nTimerID = m_vReceiver.AllocIndex();
        pReceiver = new CReceiver(*this, pfnCallback);
    }
    catch(...)
    {
        return BAD_TIMER_ID;
    }

    if (::SetTimer(m_hWnd, nTimerID, nElapse, *pReceiver) == 0)
    {
        delete pReceiver;
        return BAD_TIMER_ID;
    }

    m_vReceiver.SetAt(nTimerID, pReceiver);
    return nTimerID;
}

void CTimerHost::KillTimer(UINT nTimerID) throw()
{
    if (nTimerID < m_vReceiver.size() && m_vReceiver[nTimerID] != NULL)
    {
        CReceiver* pReceiver = m_vReceiver[nTimerID];
        m_vReceiver.SetAt(nTimerID, NULL);

        BOOL bRes = ::KillTimer(m_hWnd, nTimerID);
        _ASSERTE(bRes);
        bRes = PostMessage(m_hWnd, WM_TIMER, (WPARAM)BAD_TIMER_ID, (LPARAM)(TIMERPROC)*pReceiver);
        _ASSERTE(bRes);

        pReceiver->OnStop();

        try
        {
            UINT nID = m_vRecycle.AllocIndex();
            m_vRecycle.SetAt(nID, pReceiver);
            pReceiver->OnRecycle(nID);
        }
        catch(...)
        {
        }
    }
}

